/*
  Mário Vairinhos 2012
  This software is free. Anyone may distribute, copy or modify
  as long as he or she gives me credits.

	Modificado por Ernesto Bazzano 2018

*/

#if (defined(__AVR__) || defined(ARDUINO_ARCH_AVR))
#include <TVout.h>
#include <fontALL.h>
#include <ButtonController.h>
#include <EEPROM.h>
#include "rocket.h"
TVout TV;
#endif

#define TVSYSTEM        _PAL
#define SCREENWIDTH     128
#define SCREENHEIGHT    96
#define SCREENWIDTH2    64
#define SCREENHEIGHT2   48
#define ASTEROIDS       15

// pin controls
#define BT_UP           4
#define BT_DOWN         5
#define BT_LEFT         3
#define BT_RIGHT        2

#define BEEP      TV.tone(1046,200);

// globals
int asteroidsX[ASTEROIDS];
int asteroidsY[ASTEROIDS];
int asteroidsZ[ASTEROIDS];
float meX, meY, meZ = 0;
float screenX, screenY;
float myspeed;
int level;
int energy;
int tick;
int secondx;
int frames;
int gameState = 0;//(0-startLevel,1-run,2-gameOver,3 win)
int mover = 0;

void drawLevel()
{
  int x, y;
  x = 28;
  y = 60;
  TV.select_font(font8x8);
  TV.print(x + 11, y, "NIVEL ");
  TV.print(level);
  TV.select_font(font4x6);
  TV.print(x + 10, y + 10, level == 0 ? "Inicia el juego" : "mas asteroides!");
  TV.print(x + 30, y + 18, "<- ->");
}


void setup()
{


  //init TV library
  TV.begin(TVSYSTEM, SCREENWIDTH, SCREENHEIGHT);
 	TV.select_font(font4x6);
  TV.delay_frame(60);
#if (defined(__AVR__) || defined(ARDUINO_ARCH_AVR))
  randomSeed(analogRead(0));
  // setup pullup resistors
  pinMode(BT_UP   , INPUT);
  digitalWrite(BT_UP   , HIGH);
  pinMode(BT_DOWN , INPUT);
  digitalWrite(BT_DOWN , HIGH);
  pinMode(BT_LEFT , INPUT);
  digitalWrite(BT_LEFT , HIGH);
  pinMode(BT_RIGHT, INPUT);
  digitalWrite(BT_RIGHT, HIGH);
#endif
  level = 0;
  gameState = 0;
  //BEEP
	for (int f = 0; f < 150; f++) startLevel();
  // placa de inicio de la capi
	int y = 0;
  for (int t = 0; t < 50; t++) {
		y = random(2);
    TV.bitmap(random(2), y, intro, 3, SCREENWIDTH, SCREENHEIGHT-y);
    TV.delay(100);
  }

  TV.clear_screen();
}




// static vars
int startLev = 0;
int ry, rdy;

void startLevel()
{
  // random asteroids once
  if (startLev == 0)
  {
    BEEP
    meX = 100;
    meY = 0;
    meZ = 0;
    startLev = 1;
    ry = 20;
    rdy = 1;
    for (int i = 0; i < ASTEROIDS; i++)
    {
      asteroidsX[i] = random(130, 230);
      asteroidsY[i] = random(1, 95);
      asteroidsZ[i] = random(1, 5);
    }
  }

  // render the screen
  // and move the rocket
  tempo();
  frames++;
  if (frames > 2)
  {
    TV.clear_screen();
    frames = 0;
    ry += rdy;
    if (ry > 35) rdy = -1;
    if (ry < 25) rdy = 1;
    TV.bitmap(25 + random(-1, 1), ry + random(-1, 1), rocket, 3, 72, 24);
    // se podria agregar una distorcion al final del dibujo
    drawLevel();
  }


  // move the asteroids
  for (int i = 0; i < 10; i++)
  {
    TV.set_pixel(asteroidsX[i], asteroidsY[i], 2);
    asteroidsX[i] -= asteroidsZ[i];
    if (asteroidsX[i] <= 1) asteroidsX[i] = 130;
    if (asteroidsX[i] < 128 && asteroidsX[i] > 0) TV.set_pixel(asteroidsX[i], asteroidsY[i], 1);
  }

  // check user input
  if (Controller.rightPressed() &&  Controller.leftPressed())
  {
    initLevel();
    gameState = 1;
    startLev = 0;
    clicka();
  }

  // after 60 secondx on inactivity, start the level anyway
  //  if (secondx > 60) initLevel();

  TV.delay_frame(1);
}




void clicka()
{
  BEEP
  TV.delay(200);
  TV.invert();
  TV.delay(200);
  TV.clear_screen();
}






void win()
{
  level++;
  gameState = 0;
  startLev = 0;

  TV.clear_screen();
  // segunda placa de la capi
  TV.bitmap(0, 0, wine,3,SCREENWIDTH, SCREENHEIGHT);
  TV.delay(3000);
  // falta ponerle algo como de contenta por terminar la mision
}

void run()
{

  tempo();

  // every 3 frames clear te screen
  // and render the cockpit
  // every frame render the asteroids
  frames++;
  if (frames > 2)
  {
    frames = 0;
    TV.clear_screen();
		drawCockpit(mover);
		mover = 0;
    if (gameState == 2)
    {
      drawGameOver();
      if (secondx > 10)
      {
        gameState = 0;
        level = 0;
        TV.clear_screen();
        return;
      }
    }
  }

  if (gameState == 1)
  {
    // user input
    if (Controller.leftPressed())
    {
      if ( random (1, 50000) == 1) energy--;
      meX -= 1 + level / 3;
    }
    if (Controller.rightPressed())
    {
      if ( random (1, 50000) == 1) energy--;
      meX += 1 + level / 3;
    }
    if (Controller.upPressed())
    {
      if ( random (1, 50000) == 1) energy--;
      meY -= 1 + level / 3;
    }
    if (Controller.downPressed())
    {
      if ( random (1, 50000) == 1) energy--;
      meY += 1 + level / 3;
    }
  }
  // collision
  for (int i = 0; i < ASTEROIDS; i++)
  {
    if (asteroidsZ[i] < 20)
    {


     	if (asteroidsZ[i] < 24) mover = 1;

      // cartel de alerta
      TV.select_font(font4x6);
      TV.print(128 - (4 * 7), 1, "alerta");

      meX += random(-1, 1);
      meY += random(-1, 1);
      if (abs(asteroidsX[i] - meX) + abs(asteroidsY[i] - meY) <= 25)
      {

        if (gameState == 1)
        {

          beepCrash();
          TV.fill(2);
          energy--;

          // sacude la nave
          meX += random(-10, 10) * 2;
          meY += random(-10, 10) * 2;
        }
      }

      respawn(i, meX, meY, 800, i * 20 + 5, 40);
    }
    asteroidsZ[i] -= myspeed;
  }
  renderAsteroids();
  /* for (int y = SCREENHEIGHT/2; y < SCREENHEIGHT; y+=4 ) {
      for (int x = 0; x < SCREENWIDTH; x+=4 ) {
          // sombra
          TV.set_pixel(x, y, 1);
      }
    } */
  if (energy <= 0)
  {
    gameState = 2;
    energy = 1;
    secondx = 0;
  }


  if (secondx > 100)
  {
    win();
  }
}











void initLevel()
{
  TV.noTone();
  myspeed = 10 + level * 2; //3
  energy = 10;
  tick = 0;
  secondx = 0;
  meX = 0;
  meY = 0;
  meZ = 0;
  for (int i = 0; i < ASTEROIDS; i++)
  {
    respawn(i, meX, meY, 1000, 120, 600);
  }
  gameState = 1;
  startLev = 0;
}


void respawn(int index, int ox, int oy, int oz, int radius, int radiusZ)
{
  // asteroides en 3d
  asteroidsX[index] = ox + random(-radius*2, +radius*2);
  asteroidsY[index] = oy + random(-radius*2, +radius*2);
  asteroidsZ[index] = oz + random(radiusZ);
}
void get2dFrom3d(float coordX, float coordY, float coordZ)
{
  coordX = coordX - meX;
  coordY = coordY - meY;
  coordZ = coordZ - meZ;
  if (coordZ == 0) coordZ = 0.1;
  screenX = (float(coordX) / float(coordZ)) * SCREENWIDTH;
  screenY = (float(coordY) / float(coordZ)) * SCREENHEIGHT;
}

void renderAsteroids()
{
  float x1, radius;
  int vx, vy;
  for (int i = 0; i < ASTEROIDS; i++)
  {
    get2dFrom3d(asteroidsX[i], asteroidsY[i], asteroidsZ[i]);
    radius = 2200 / (asteroidsZ[i] << 2);
    vx = SCREENWIDTH2 + screenX;
    vy = SCREENHEIGHT2 + screenY;
    if (vx < (SCREENWIDTH - radius - 1) && vx > (radius + 1) && vy < (SCREENHEIGHT - radius - 1) && vy > (radius + 1))
    {
      TV.draw_circle(vx, vy, radius, 1, 1);
      if ( random(0, 2) == 1) TV.draw_circle(vx + random(-3, 3), vy + random(-3, 3), radius / random(2, 3), 0, 0);
    }
  }

  TV.delay_frame(1);
}

void tempo() {
  tick++;
  if (tick > 30) {
    tick = 0;
    secondx++;
  }
}

// me gustaria rotar el screen de la pantalla

void drawCockpit(int a)
{
	int b = 0;
	if (a == 1) {
			a = rand() % 2;
			b = rand() % 2;
	}
  TV.select_font(font4x6);
  int x = 43 + a;
  int y = 71 + b;
  TV.print_char(x, y, 'E');
  TV.draw_rect (x + 6, y + 1, energy, 4  , 1, 1);
  TV.draw_rect (x + 5, y + 0, 12 , 5  , 1);
  x = 65 + a;
  y = 71 + b;
  TV.print_char(x, y, 'T');
  TV.draw_rect (x + 6, y + 1 , secondx / 10  , 4  , 1, 1);
  TV.draw_rect (x + 5, y   , 12 , 5  , 1);

  TV.draw_line(20 + a, SCREENHEIGHT - b, 40, SCREENHEIGHT - 16, 1);
  TV.draw_line(SCREENWIDTH - 20 + a, SCREENHEIGHT - b, SCREENWIDTH - 40, SCREENHEIGHT - 16, 1);
  TV.draw_rect(40 + a, SCREENHEIGHT - 16 - 12 - b, SCREENWIDTH - 40 - 40, 12, 1);
  if (energy > 2) TV.draw_line(0 + a, SCREENHEIGHT - 16 - 12 - 30 + b, 40, SCREENHEIGHT - 16 - 12, 1);
  if (energy > 3) TV.draw_line(SCREENWIDTH - a, SCREENHEIGHT - 16 - 12 - 30 - b, SCREENWIDTH - 40, SCREENHEIGHT - 16 - 12, 1);

  // panel de la capi
  // agregar caras de muy feliz, feliz, normal, triste, dolorida, muerta
  // separador de que se vea en navecita por que pediste
  // TV.draw_rect(SCREENWIDTH * .4 , SCREENHEIGHT * .4, SCREENWIDTH * .8, SCREENHEIGHT * .8, 1);
  //TV.bitmap(SCREENWIDTH-40-a,SCREENHEIGHT-42-b, cara, 0, 40, 42);
  x = 55 + a;
  y = 84 + b;
  TV.print_char(x, y, 'N');
  TV.draw_rect (x + 6, y + 1 , level / 3  , 4  , 1, 1);
  TV.draw_rect (x + 5, y   , 12 , 5  , 1);
  int humor = ((3 + (secondx / -30)) + (energy / -8)) + 1;
  if (a > 0) humor = humor + 1;
  if (humor < 0) humor = 0;
  if (humor > 4) humor = 4;
  TV.bitmap((128 / 4) + 11 + (a / 2), 3 + b, cara, 3 + (40 / 8 * 19 * humor), 40, 19);

  //        TV.bitmap(25+ random(-1, 1), ry + random(-1, 1), rocket,3,72,24);

}

void drawGameOver()
{
  TV.select_font(font8x8);
  TV.print(30, 30, "GAME OVER");
  TV.delay_frame(1);
}



void beepCrash()
{
  TV.tone(random(60, 100), 150);
}


// game's state
void loop()
{
  switch (gameState)
  {
    case 0:
      startLevel();
      break;
    case 1:
      run();
      break;
    case 2:
      run();
      break;
    case 3:
      win();
      break;
  }
}
